@@ -19,11 +19,11 @@ class KerasExportSerializationFormat(Enum):
     """
     Specify which serialization format to use for exporting a quantized Keras model.
 
-    KERAS_H5 - .keras (TF2.13 and above) or .h5 (TF2.12 and below) file format
+    KERAS - .keras file format
 
     TFLITE - .tflite file format
 
     """
 
-    KERAS_H5 = 0
+    KERAS = 0
     TFLITE = 1
