@@ -15,12 +15,9 @@
 from typing import Callable, Dict
 
 from model_compression_toolkit.constants import FOUND_TF
-from model_compression_toolkit.exporter.model_exporter.keras.export_serialization_format import \
-    KerasExportSerializationFormat
+from model_compression_toolkit.exporter.model_exporter.fw_agonstic.quantization_format import QuantizationFormat
 from model_compression_toolkit.logger import Logger
-from model_compression_toolkit.target_platform_capabilities.target_platform import TargetPlatformCapabilities
-from model_compression_toolkit.target_platform_capabilities.target_platform.quantization_format import \
-    QuantizationFormat
+
 
 if FOUND_TF:
     import keras
@@ -30,64 +27,67 @@ if FOUND_TF:
     from model_compression_toolkit.exporter.model_exporter.keras.fakely_quant_tflite_exporter import \
         FakelyQuantTFLiteExporter
     from model_compression_toolkit.exporter.model_exporter.keras.int8_tflite_exporter import INT8TFLiteExporter
+    from model_compression_toolkit.exporter.model_exporter.keras.export_serialization_format import \
+        KerasExportSerializationFormat
+    from model_compression_toolkit.exporter.model_exporter.keras.mctq_keras_exporter import MCTQKerasExporter
 
     supported_serialization_quantization_export_dict = {
-        KerasExportSerializationFormat.KERAS_H5: [QuantizationFormat.FAKELY_QUANT],
+        KerasExportSerializationFormat.KERAS: [QuantizationFormat.FAKELY_QUANT, QuantizationFormat.MCTQ],
         KerasExportSerializationFormat.TFLITE: [QuantizationFormat.FAKELY_QUANT, QuantizationFormat.INT8]
     }
 
     def keras_export_model(model: keras.models.Model,
                            save_model_path: str,
-                           target_platform_capabilities: TargetPlatformCapabilities,
                            is_layer_exportable_fn: Callable = is_keras_layer_exportable,
-                           serialization_format: KerasExportSerializationFormat = KerasExportSerializationFormat.KERAS_H5) -> Dict[str, type]:
+                           serialization_format: KerasExportSerializationFormat = KerasExportSerializationFormat.KERAS,
+                           quantization_format : QuantizationFormat = QuantizationFormat.MCTQ) -> Dict[str, type]:
         """
-        Export a Keras quantized model to a h5 or tflite model.
+        Export a Keras quantized model to a .keras or .tflite format model (according to serialization_format).
         The model will be saved to the path in save_model_path.
-        keras_export_model supports the combination of QuantizationFormat.FAKELY_QUANT (where weights
-        and activations are float fakely-quantized values) and KerasExportSerializationFormat.KERAS_H5 (where the model
-        will be saved to h5 model) or the combination of KerasExportSerializationFormat.TFLITE (where the model will be
-        saved to tflite model) with QuantizationFormat.FAKELY_QUANT or QuantizationFormat.INT8 (where weights and
-        activations are represented using 8bits integers).
+        Models that are exported to .keras format can use quantization_format of QuantizationFormat.MCTQ or QuantizationFormat.FAKELY_QUANT.
+        Models that are exported to .tflite format can use quantization_format of QuantizationFormat.INT8 or QuantizationFormat.FAKELY_QUANT.
 
         Args:
             model: Model to export.
             save_model_path: Path to save the model.
-            target_platform_capabilities: TargetPlatformCapabilities object that describes the desired inference
-            target platform (includes quantization format).
             is_layer_exportable_fn: Callable to check whether a layer can be exported or not.
-            serialization_format: Format to export the model according to (by default
-            KerasExportSerializationFormat.KERAS_H5).
+            serialization_format: Format to export the model according to (KerasExportSerializationFormat.KERAS, by default).
+            quantization_format: Format of how quantizers are exported (MCTQ quantizers, by default).
 
         Returns:
             Custom objects dictionary needed to load the model.
 
         """
 
-        if serialization_format == KerasExportSerializationFormat.KERAS_H5:
-            if target_platform_capabilities.tp_model.quantization_format == QuantizationFormat.FAKELY_QUANT:
+        if serialization_format == KerasExportSerializationFormat.KERAS:
+            if quantization_format == QuantizationFormat.FAKELY_QUANT:
                 exporter = FakelyQuantKerasExporter(model,
                                                     is_layer_exportable_fn,
                                                     save_model_path)
+            elif quantization_format == QuantizationFormat.MCTQ:
+                exporter = MCTQKerasExporter(model,
+                                             is_layer_exportable_fn,
+                                             save_model_path)
+
             else:
                 Logger.critical(
-                    f'Unsupported quantization {target_platform_capabilities.tp_model.quantization_format} for '
+                    f'Unsupported quantization {quantization_format} for '
                     f'serialization {serialization_format} was used to export Keras model. Please see API for '
                     f'supported formats.')  # pragma: no cover
 
         elif serialization_format == KerasExportSerializationFormat.TFLITE:
-            if target_platform_capabilities.tp_model.quantization_format == QuantizationFormat.FAKELY_QUANT:
+            if quantization_format == QuantizationFormat.FAKELY_QUANT:
                 exporter = FakelyQuantTFLiteExporter(model,
                                                      is_layer_exportable_fn,
                                                      save_model_path)
 
-            elif target_platform_capabilities.tp_model.quantization_format == QuantizationFormat.INT8:
+            elif quantization_format == QuantizationFormat.INT8:
                 exporter = INT8TFLiteExporter(model,
                                               is_layer_exportable_fn,
                                               save_model_path)
             else:
                 Logger.critical(
-                    f'Unsupported quantization {target_platform_capabilities.tp_model.quantization_format} for '
+                    f'Unsupported quantization {quantization_format} for '
                     f'serialization {serialization_format} was used to export Keras model. Please see API for '
                     f'supported formats.')  # pragma: no cover
 
@@ -101,6 +101,5 @@ if FOUND_TF:
         return exporter.get_custom_objects()
 else:
     def keras_export_model(*args, **kwargs):
-        Logger.error('Installing tensorflow is mandatory '
-                     'when using keras_export_model. '
-                     'Could not find some or all of TensorFlow packages.')  # pragma: no cover
+        Logger.critical("Tensorflow must be installed to use keras_export_model. "
+                        "The 'tensorflow' package is missing.")  # pragma: no cover
