@@ -12,5 +12,5 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-
-from model_compression_toolkit.qat.common.constants import THRESHOLD_TENSOR, WEIGHTS_QUANTIZATION_PARAMS
\ No newline at end of file
+from model_compression_toolkit.trainable_infrastructure.common.constants import THRESHOLD_TENSOR, \
+    WEIGHTS_QUANTIZATION_PARAMS
