@@ -24,6 +24,7 @@ def is_qat_applicable(node: common.BaseNode,
                       fw_info: FrameworkInfo) -> bool:
     """
     A function for deciding if a layer should be fine-tuned during QAT
+
     Args:
         node (BaseNode): Node for quantization decision
         fw_info (FrameworkInfo): Pytorch quantization information
@@ -32,9 +33,10 @@ def is_qat_applicable(node: common.BaseNode,
         A boolean whether the layer is to be wrapped with a QuantizeWrapper
     """
 
-    if node.is_weights_quantization_enabled() and not fw_info.is_kernel_op(node.type):
-        Logger.error("QAT Error: Quantizing a node without a kernel isn't supported")
-    return node.is_weights_quantization_enabled() or node.is_activation_quantization_enabled()
+    kernel_attr = fw_info.get_kernel_op_attributes(node.type)[0]
+    return (kernel_attr is not None and node.is_weights_quantization_enabled(kernel_attr)) \
+            or node.is_activation_quantization_enabled()
+
 
 
 class TrainingMethod(Enum):
@@ -45,9 +47,12 @@ class TrainingMethod(Enum):
 
     DQA -  DNN Quantization with Attention. Includes a smooth quantization introduces by DQA method
 
+    LSQ - Learned Step size Quantization. Includes PowerOfTwo, symmetric & uniform quantizers: https://arxiv.org/pdf/1902.08153.pdf
+
     """
     STE = "STE",
-    DQA = "DQA"
+    DQA = "DQA",
+    LSQ = "LSQ"
 
 
 class QATConfig:
